/*
 * thechess, chess game web application written in C++ and based on Wt
 * Copyright (C) 2010 Boris Nagaev
 *
 * See the LICENSE file for terms of use.
 */

#ifndef THECHESS_MAIN_WIDGET_HPP_
#define THECHESS_MAIN_WIDGET_HPP_

#include <Wt/WGlobal>
#include <Wt/WContainerWidget>
#include <Wt/Wc/global.hpp>

#include "thechess-global.hpp"
#include "model/global.hpp"
#include "widgets/global.hpp"

namespace thechess {

/** Main Application widget */
class MainWidget : public Wt::WContainerWidget {
public:
    /** Constructor */
    MainWidget(Wt::WContainerWidget* parent = 0);

    /** Show the menu with references */
    void show_menu(Path* path);

    /** Get main menu */
    MainMenu* main_menu();

    /** Update list of user's games (East) */
    void update_my_games();

    /** List of users */
    void user_list();

    /** View the user */
    void user_view(const UserPtr& user);

    /** View user's virtuals */
    void virtuals_of_user(const UserPtr& user);

    /** View user's games */
    void games_of_user(const UserPtr& user);

    /** View user's competitions */
    void competitions_of_user(const UserPtr& user);

    /** View all virtuals */
    void all_virtuals();

    /** Settings page */
    void settings_page();

    /** Create update-password dialog */
    void update_password();

    /** List of games */
    void game_list();

    /** View the game */
    void game_view(const GamePtr& game);

    /** Create new game */
    void game_new();

    /** List of game parameters */
    void gp_list();

    /** List of competitions */
    void competition_list();

    /** View the competition */
    void competition_view(const CompetitionPtr& competition);

    /** Create new competition */
    void competition_new();

    /** View the board */
    void board_view(const std::string& data);

    /** View forum topics */
    void forum_topics();

    /** View posts of this topic */
    void forum_topic_posts(const CommentPtr& topic);

    /** View all posts */
    void forum_all_posts();

    /** View post */
    void forum_post(const CommentPtr& post);

    /** View post comment */
    void forum_post_comment(const CommentPtr& comment);

    /** Edit forum post */
    void forum_edit(const CommentPtr& post_or_text);

    /** View chat comment */
    void chat_comment(const CommentPtr& comment);

    /** Get authentication widget.
    \attention Auth widget should be set (by Application).
    */
    Wt::Auth::AuthWidget* auth_widget();

    /** Set authentication widget */
    void set_auth_widget(Wt::Auth::AuthWidget* widget);

    /** Get SWF store.
    \attention The widget must be set (by Application).
    */
    Wt::Wc::SWFStore* swf_store();

    /** Set swf store.
    Ownership of the swf store is transferred.
    Do not call this method twice.
    */
    void set_swfstore(Wt::Wc::SWFStore* swfstore);

private:
    Wt::WContainerWidget* auth_place_;
    Wt::WContainerWidget* menu_place_;
    Wt::WContainerWidget* contents_place_;
    Wt::WContainerWidget* mygames_place_;
    Wt::WContainerWidget* bottom_place_;

    void set_contents(WWidget* widget);
};

}

#endif

